@@ -76,6 +76,8 @@
 /* Delay before updating the widget when the id entry changed (seconds) */
 #define ID_CHANGED_TIMEOUT 1
 
+#define DATA_FIELD "contact-info-field"
+
 typedef struct
 {
   EmpathyContactManager *manager;
@@ -123,6 +125,7 @@ typedef struct
   GtkWidget *spinner_details;
   GList *details_to_set;
   GCancellable *details_cancellable;
+  gboolean details_changed;
 
   /* Client */
   GtkWidget *vbox_client;
@@ -207,8 +210,12 @@ contact_widget_save (EmpathyContactWidget *information)
 
   if (information->details_to_set != NULL)
     {
-      tp_connection_set_contact_info_async (connection,
-          information->details_to_set, set_contact_info_cb, NULL);
+      if (information->details_changed)
+        {
+          tp_connection_set_contact_info_async (connection,
+              information->details_to_set, set_contact_info_cb, NULL);
+        }
+
       tp_contact_info_list_free (information->details_to_set);
       information->details_to_set = NULL;
     }
@@ -227,9 +234,15 @@ contact_widget_details_setup (EmpathyContactWidget *information)
 
 static void
 contact_widget_details_changed_cb (GtkEntry *entry,
-    TpContactInfoField *field)
+    EmpathyContactWidget *self)
 {
   const gchar *strv[] = { NULL, NULL };
+  TpContactInfoField *field;
+
+  self->details_changed = TRUE;
+
+  field = g_object_get_data ((GObject *) entry, DATA_FIELD);
+  g_assert (field != NULL);
 
   strv[0] = gtk_entry_get_text (entry);
 
@@ -240,12 +253,18 @@ contact_widget_details_changed_cb (GtkEntry *entry,
 
 static void
 contact_widget_bday_changed_cb (GtkCalendar *calendar,
-    TpContactInfoField *field)
+    EmpathyContactWidget *self)
 {
   guint year, month, day;
   GDate *date;
   gchar tmp[255];
   const gchar *strv[] = { NULL, NULL };
+  TpContactInfoField *field;
+
+  self->details_changed = TRUE;
+
+  field = g_object_get_data ((GObject *) calendar, DATA_FIELD);
+  g_assert (field != NULL);
 
   gtk_calendar_get_date (calendar, &year, &month, &day);
   date = g_date_new_dmy (day, month+1, year);
@@ -368,6 +387,8 @@ contact_widget_details_update_edit (EmpathyContactWidget *information)
 
   g_assert (information->details_to_set == NULL);
 
+  information->details_changed = FALSE;
+
   contact = empathy_contact_get_tp_contact (information->contact);
   connection = tp_contact_get_connection (contact);
 
@@ -420,6 +441,7 @@ contact_widget_details_update_edit (EmpathyContactWidget *information)
       TpContactInfoField *field = l->data;
       InfoFieldData *field_data;
       GtkWidget *w;
+      TpContactInfoFieldSpec *spec;
 
       field_data = find_info_field_data (field->field_name);
       if (field_data == NULL)
@@ -431,6 +453,18 @@ contact_widget_details_update_edit (EmpathyContactWidget *information)
           continue;
         }
 
+      spec = get_spec_from_list (specs, field->field_name);
+      /* We shouldn't have added the field to details_to_set if it's not
+       * supported by the CM */
+      g_assert (spec != NULL);
+
+      if (spec->flags & TP_CONTACT_INFO_FIELD_FLAG_OVERWRITTEN_BY_NICKNAME)
+        {
+          DEBUG ("Ignoring field '%s' due it to having the "
+              "Overwritten_By_Nickname flag", field->field_name);
+          continue;
+        }
+
       /* Add Title */
       w = gtk_label_new (_(field_data->title));
       gtk_table_attach (GTK_TABLE (information->table_details),
@@ -461,10 +495,12 @@ contact_widget_details_update_edit (EmpathyContactWidget *information)
               w, 1, 2, n_rows, n_rows + 1);
           gtk_widget_show (w);
 
+          g_object_set_data ((GObject *) w, DATA_FIELD, field);
+
           g_signal_connect (w, "day-selected-double-click",
-            G_CALLBACK (contact_widget_bday_changed_cb), field);
+            G_CALLBACK (contact_widget_bday_changed_cb), information);
           g_signal_connect (w, "month-changed",
-            G_CALLBACK (contact_widget_bday_changed_cb), field);
+            G_CALLBACK (contact_widget_bday_changed_cb), information);
         }
       else
         {
@@ -475,8 +511,10 @@ contact_widget_details_update_edit (EmpathyContactWidget *information)
               w, 1, 2, n_rows, n_rows + 1);
           gtk_widget_show (w);
 
+          g_object_set_data ((GObject *) w, DATA_FIELD, field);
+
           g_signal_connect (w, "changed",
-            G_CALLBACK (contact_widget_details_changed_cb), field);
+            G_CALLBACK (contact_widget_details_changed_cb), information);
         }
 
       n_rows++;
@@ -701,11 +739,11 @@ contact_widget_groups_update (EmpathyContactWidget *information)
       FolksPersona *persona =
           empathy_contact_get_persona (information->contact);
 
-      if (FOLKS_IS_GROUPABLE (persona))
+      if (FOLKS_IS_GROUP_DETAILS (persona))
         {
-          empathy_groups_widget_set_groupable (
+          empathy_groups_widget_set_group_details (
               EMPATHY_GROUPS_WIDGET (information->groups_widget),
-              FOLKS_GROUPABLE (persona));
+              FOLKS_GROUP_DETAILS (persona));
           gtk_widget_show (information->groups_widget);
 
           return;
@@ -1176,13 +1214,44 @@ set_nickname_cb (GObject *source,
     }
 }
 
+/* Update all the contact info fields having the
+* Overwritten_By_Nickname flag to the new alias. This avoid accidentally
+* reseting the alias when calling SetContactInfo(). See bgo #644298 for
+* details. */
 static void
-set_alias_on_account (TpAccount *account,
-    const gchar *alias)
+update_nickname_in_contact_info (EmpathyContactWidget *self,
+    TpConnection *connection,
+    const gchar *nickname)
 {
-  DEBUG ("Set Account.Nickname to %s", alias);
+  GList *specs, *l;
 
-  tp_account_set_nickname_async (account, alias, set_nickname_cb, NULL);
+  specs = tp_connection_get_contact_info_supported_fields (connection);
+
+  for (l = self->details_to_set; l != NULL; l= g_list_next (l))
+    {
+      TpContactInfoField *field = l->data;
+      TpContactInfoFieldSpec *spec;
+
+      spec = get_spec_from_list (specs, field->field_name);
+      /* We shouldn't have added the field to details_to_set if it's not
+       * supported by the CM */
+      g_assert (spec != NULL);
+
+      if (spec->flags & TP_CONTACT_INFO_FIELD_FLAG_OVERWRITTEN_BY_NICKNAME)
+        {
+          const gchar *strv[] = { nickname, NULL };
+
+          DEBUG ("Updating field '%s' to '%s' as it has the "
+              "Overwritten_By_Nickname flag and Account.Nickname has "
+              "been updated", field->field_name, nickname);
+
+          if (field->field_value != NULL)
+            g_strfreev (field->field_value);
+          field->field_value = g_strdupv ((GStrv) strv);
+        }
+    }
+
+  g_list_free (specs);
 }
 
 static gboolean
@@ -1199,9 +1268,21 @@ contact_widget_entry_alias_focus_event_cb (GtkEditable *editable,
       if (empathy_contact_is_user (information->contact))
         {
           TpAccount * account;
+          const gchar *current_nickname;
 
           account = empathy_contact_get_account (information->contact);
-          set_alias_on_account (account, alias);
+          current_nickname = tp_account_get_nickname (account);
+
+          if (tp_strdiff (current_nickname, alias))
+            {
+              DEBUG ("Set Account.Nickname to %s", alias);
+
+              tp_account_set_nickname_async (account, alias, set_nickname_cb,
+                  NULL);
+
+              update_nickname_in_contact_info (information,
+                  empathy_contact_get_connection (information->contact), alias);
+            }
         }
       else
         {
@@ -1396,10 +1477,10 @@ contact_widget_contact_update (EmpathyContactWidget *information)
           FolksPersona *persona = empathy_contact_get_persona (
               information->contact);
 
-          if (persona != NULL && FOLKS_IS_FAVOURITABLE (persona))
+          if (persona != NULL && FOLKS_IS_FAVOURITE_DETAILS (persona))
             {
-              gboolean is_favourite = folks_favouritable_get_is_favourite (
-                  FOLKS_FAVOURITABLE (persona));
+              gboolean is_favourite = folks_favourite_details_get_is_favourite (
+                  FOLKS_FAVOURITE_DETAILS (persona));
               contact_widget_favourites_changed_cb (information->manager,
                   information->contact, is_favourite, information);
             }
@@ -1530,10 +1611,11 @@ favourite_toggled_cb (GtkToggleButton *button,
 {
   FolksPersona *persona = empathy_contact_get_persona (information->contact);
 
-  if (persona != NULL && FOLKS_IS_FAVOURITABLE (persona))
+  if (persona != NULL && FOLKS_IS_FAVOURITE_DETAILS (persona))
     {
       gboolean active = gtk_toggle_button_get_active (button);
-      folks_favouritable_set_is_favourite (FOLKS_FAVOURITABLE (persona), active);
+      folks_favourite_details_set_is_favourite (
+          FOLKS_FAVOURITE_DETAILS (persona), active);
     }
 }
 
