@@ -45,6 +45,7 @@
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-contact-list.h>
+#include <libempathy/empathy-request-util.h>
 
 #include <libempathy-gtk/empathy-images.h>
 #include <libempathy-gtk/empathy-contact-dialogs.h>
@@ -59,6 +60,7 @@
 #include "empathy-chat-window.h"
 #include "empathy-about-dialog.h"
 #include "empathy-invite-participant-dialog.h"
+#include "gedit-close-button.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_CHAT
 #include <libempathy/empathy-debug.h>
@@ -148,7 +150,8 @@ static const GtkTargetEntry drag_types_dest_file[] = {
 	{ "text/uri-list", 0, DND_DRAG_TYPE_URI_LIST },
 };
 
-static void chat_window_update (EmpathyChatWindow *window);
+static void chat_window_update (EmpathyChatWindow *window,
+		gboolean update_contact_menu);
 
 static void empathy_chat_window_add_chat (EmpathyChatWindow *window,
 			      EmpathyChat	*chat);
@@ -260,8 +263,6 @@ chat_window_create_label (EmpathyChatWindow *window,
 	GtkWidget            *hbox;
 	GtkWidget            *name_label;
 	GtkWidget            *status_image;
-	GtkWidget            *close_button;
-	GtkWidget            *close_image;
 	GtkWidget            *event_box;
 	GtkWidget            *event_box_hbox;
 	PangoAttrList        *attr_list;
@@ -312,8 +313,9 @@ chat_window_create_label (EmpathyChatWindow *window,
 	gtk_box_pack_start (GTK_BOX (hbox), event_box, TRUE, TRUE, 0);
 
 	if (is_tab_label) {
-		close_button = gtk_button_new ();
-		gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
+		GtkWidget            *close_button;
+
+		close_button = gedit_close_button_new ();
 		g_object_set_data (G_OBJECT (chat), "chat-window-tab-close-button", close_button);
 
 		/* We don't want focus/keynav for the button to avoid clutter, and
@@ -322,13 +324,6 @@ chat_window_create_label (EmpathyChatWindow *window,
 		gtk_widget_set_can_focus (close_button, FALSE);
 		gtk_widget_set_can_default (close_button, FALSE);
 
-		/* Set the name to make the special rc style match. */
-		gtk_widget_set_name (close_button, "empathy-close-button");
-
-		close_image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
-
-		gtk_container_add (GTK_CONTAINER (close_button), close_image);
-
 		gtk_box_pack_end (GTK_BOX (hbox), close_button, FALSE, FALSE, 0);
 
 		g_signal_connect (close_button,
@@ -356,7 +351,7 @@ _submenu_notify_visible_changed_cb (GObject    *object,
 	g_signal_handlers_disconnect_by_func (object,
 					      _submenu_notify_visible_changed_cb,
 					      userdata);
-	chat_window_update (EMPATHY_CHAT_WINDOW (userdata));
+	chat_window_update (EMPATHY_CHAT_WINDOW (userdata), TRUE);
 }
 
 static void
@@ -424,12 +419,15 @@ chat_window_contact_menu_update (EmpathyChatWindowPriv *priv,
 
 	if (orig_submenu == NULL || !gtk_widget_get_visible (orig_submenu)) {
 		submenu = empathy_chat_get_contact_menu (priv->current_chat);
+
 		if (submenu != NULL) {
+			/* gtk_menu_attach_to_widget () doesn't behave nicely here */
+			g_object_set_data (G_OBJECT (submenu), "window", priv->dialog);
+
 			gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), submenu);
 			gtk_widget_show (menu);
 			gtk_widget_set_sensitive (menu, TRUE);
-		}
-		else {
+		} else {
 			gtk_widget_set_sensitive (menu, FALSE);
 		}
 	} else {
@@ -584,7 +582,8 @@ chat_window_close_button_update (EmpathyChatWindowPriv *priv,
 }
 
 static void
-chat_window_update (EmpathyChatWindow *window)
+chat_window_update (EmpathyChatWindow *window,
+		    gboolean update_contact_menu)
 {
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
 	gint                   num_pages;
@@ -597,8 +596,14 @@ chat_window_update (EmpathyChatWindow *window)
 
 	chat_window_conversation_menu_update (priv, window);
 
-	chat_window_contact_menu_update (priv,
-					 window);
+	/* If this update is due to a focus-in event, we know the menu will be
+	   the same as when we last left it, so no work to do.  Besides, if we
+	   swap out the menu on a focus-in, we may confuse any external global
+	   menu watching. */
+	if (update_contact_menu) {
+		chat_window_contact_menu_update (priv,
+						 window);
+	}
 
 	chat_window_title_update (priv);
 
@@ -626,7 +631,8 @@ append_markup_printf (GString    *string,
 }
 
 static void
-chat_window_update_chat_tab (EmpathyChat *chat)
+chat_window_update_chat_tab_full (EmpathyChat *chat,
+				  gboolean update_contact_menu)
 {
 	EmpathyChatWindow     *window;
 	EmpathyChatWindowPriv *priv;
@@ -730,10 +736,16 @@ chat_window_update_chat_tab (EmpathyChat *chat)
 
 	/* Update the window if it's the current chat */
 	if (priv->current_chat == chat) {
-		chat_window_update (window);
+		chat_window_update (window, update_contact_menu);
 	}
 }
 
+static void
+chat_window_update_chat_tab (EmpathyChat *chat)
+{
+	chat_window_update_chat_tab_full (chat, TRUE);
+}
+
 static void
 chat_window_chat_notify_cb (EmpathyChat *chat)
 {
@@ -1449,9 +1461,10 @@ chat_window_new_message_cb (EmpathyChat       *chat,
 	}
 
 	if (needs_urgency) {
+		chat_window_set_highlight_room_labels (chat);
+
 		if (!has_focus) {
 			chat_window_set_urgency_hint (window, TRUE);
-			chat_window_set_highlight_room_labels (chat);
 		}
 
 		/* Pending messages have already been displayed and notified in the
@@ -1621,7 +1634,7 @@ chat_window_page_removed_cb (GtkNotebook      *notebook,
 	if (priv->chats == NULL) {
 		g_object_unref (window);
 	} else {
-		chat_window_update (window);
+		chat_window_update (window, TRUE);
 	}
 }
 
@@ -1639,7 +1652,7 @@ chat_window_focus_in_event_cb (GtkWidget        *widget,
 	chat_window_set_urgency_hint (window, FALSE);
 
 	/* Update the title, since we now mark all unread messages as read. */
-	chat_window_update_chat_tab (priv->current_chat);
+	chat_window_update_chat_tab_full (priv->current_chat, FALSE);
 
 	return FALSE;
 }
@@ -1771,7 +1784,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		}
 
 		if (!chat) {
-			empathy_dispatcher_chat_with_contact_id (
+			empathy_chat_with_contact_id (
 				account, contact_id, gtk_get_current_event_time ());
 
 			g_strfreev (strv);
@@ -1910,16 +1923,6 @@ empathy_chat_window_class_init (EmpathyChatWindowClass *klass)
 	object_class->finalize = chat_window_finalize;
 
 	g_type_class_add_private (object_class, sizeof (EmpathyChatWindowPriv));
-
-	/* Set up a style for the close button with no focus padding. */
-	gtk_rc_parse_string (
-		"style \"empathy-close-button-style\"\n"
-		"{\n"
-		"  GtkWidget::focus-padding = 0\n"
-		"  xthickness = 0\n"
-		"  ythickness = 0\n"
-		"}\n"
-		"widget \"*.empathy-close-button\" style \"empathy-close-button-style\"");
 }
 
 static void
